@@ -1,6 +1,8 @@
 #ifndef OPENMPHEADER_H
 #define OPENMPHEADER_H
 
+#define EIGEN_PERMANENTLY_DISABLE_STUPID_WARNINGS 
+
 #ifdef _OPENMP
 #include <omp.h> 
 #define OMP_IS_USED true
@@ -12,7 +14,9 @@
 #define omp_get_thread_limit() 1
 #define omp_get_num_procs()    1
 #define omp_set_nested(a)   // empty statement to remove the call
+#define omp_set_num_threads(b)   // empty statement to remove the call
 #define omp_get_wtime()        0
+#define omp_set_dynamic(a) // empty statement to remove the call
 #define OMP_IS_USED false
 #endif
 
