@@ -3,18 +3,50 @@
 using namespace Rcpp;
 
 // [[Rcpp::export]]
-SEXP Model__new(SEXP y_,SEXP formula_, SEXP data_, SEXP colnames_,
-                SEXP family_, SEXP link_){
-  Eigen::VectorXd y = as<Eigen::VectorXd>(y_);
+SEXP ModelBits__new(SEXP formula_, SEXP data_, SEXP colnames_,
+                    SEXP family_, SEXP link_, SEXP beta_,
+                    SEXP theta_){
   std::string formula = as<std::string>(formula_);
   Eigen::ArrayXXd data = as<Eigen::ArrayXXd>(data_);
   std::vector<std::string> colnames = as<std::vector<std::string> >(colnames_);
   std::string family = as<std::string>(family_);
   std::string link = as<std::string>(link_);
-  XPtr<glmmr::Model> ptr(new glmmr::Model(y,formula,data,colnames,family,link),true);
+  std::vector<double> beta = as<std::vector<double> >(beta_);
+  std::vector<double> theta = as<std::vector<double> >(theta_);
+  XPtr<glmmr::ModelBits> ptr(new glmmr::ModelBits(formula,data,colnames,family,link),true);
+  ptr->linear_predictor.update_parameters(beta);
+  ptr->covariance.update_parameters(theta);
   return ptr;
 }
 
+// [[Rcpp::export]]
+void ModelBits__update_beta(SEXP xp, SEXP beta_){
+  std::vector<double> beta = as<std::vector<double> >(beta_);
+  XPtr<glmmr::ModelBits> ptr(xp);
+  ptr->linear_predictor.update_parameters(beta);
+}
+
+// [[Rcpp::export]]
+void ModelBits__update_theta(SEXP xp, SEXP theta_){
+  std::vector<double> theta = as<std::vector<double> >(theta_);
+  XPtr<glmmr::ModelBits> ptr(xp);
+  ptr->covariance.update_parameters(theta);
+}
+
+// [[Rcpp::export]]
+SEXP Model__new_from_bits(SEXP bptr_){
+  XPtr<glmmr::ModelBits> bptr(bptr_);
+  XPtr<glmmr::Model> ptr(new glmmr::Model(*bptr),true);
+  return ptr;
+}
+
+// [[Rcpp::export]]
+void Model__set_y(SEXP xp, SEXP y_){
+  Eigen::VectorXd y = as<Eigen::VectorXd>(y_);
+  XPtr<glmmr::Model> ptr(xp);
+  ptr->set_y(y);
+}
+
 // [[Rcpp::export]]
 void Model__set_offset(SEXP xp, SEXP offset_){
   Eigen::VectorXd offset = as<Eigen::VectorXd>(offset_);
@@ -33,14 +65,14 @@ void Model__set_weights(SEXP xp, SEXP weights_){
 void Model__update_beta(SEXP xp, SEXP beta_){
   std::vector<double> beta = as<std::vector<double> >(beta_);
   XPtr<glmmr::Model> ptr(xp);
-  ptr->update_beta_extern(beta);
+  ptr->update_beta(beta);
 }
 
 // [[Rcpp::export]]
 void Model__update_theta(SEXP xp, SEXP theta_){
   std::vector<double> theta = as<std::vector<double> >(theta_);
   XPtr<glmmr::Model> ptr(xp);
-  ptr->update_theta_extern(theta);
+  ptr->update_theta(theta);
 }
 
 // [[Rcpp::export]]
@@ -54,19 +86,19 @@ void Model__update_u(SEXP xp, SEXP u_){
 void Model__use_attenuation(SEXP xp, SEXP use_){
   bool use = as<bool>(use_);
   XPtr<glmmr::Model> ptr(xp);
-  ptr->attenuate_ = use;
+  ptr->matrix.W.attenuated = use;
 }
 
 // [[Rcpp::export]]
 void Model__update_W(SEXP xp){
   XPtr<glmmr::Model> ptr(xp);
-  ptr->update_W();
+  ptr->matrix.W.update();
 }
 
 // [[Rcpp::export]]
 SEXP Model__get_W(SEXP xp){
   XPtr<glmmr::Model> ptr(xp);
-  VectorXd W = ptr->W();
+  VectorXd W = ptr->matrix.W.W();
   return wrap(W);
 }
 
@@ -74,7 +106,7 @@ SEXP Model__get_W(SEXP xp){
 SEXP Model__log_prob(SEXP xp, SEXP v_){
   Eigen::VectorXd v = as<Eigen::VectorXd>(v_);
   XPtr<glmmr::Model> ptr(xp);
-  double logprob = ptr->log_prob(v);
+  double logprob = ptr->mcmc.log_prob(v);
   return wrap(logprob);
 }
 
@@ -83,125 +115,125 @@ SEXP Model__log_gradient(SEXP xp, SEXP v_, SEXP beta_){
   Eigen::VectorXd v = as<Eigen::VectorXd>(v_);
   bool beta = as<bool>(beta_);
   XPtr<glmmr::Model> ptr(xp);
-  Eigen::VectorXd loggrad = ptr->log_gradient(v,beta);
+  Eigen::VectorXd loggrad = ptr->matrix.log_gradient(v,beta);
   return wrap(loggrad);
 }
 
 // [[Rcpp::export]]
 SEXP Model__linear_predictor(SEXP xp){
   XPtr<glmmr::Model> ptr(xp);
-  Eigen::MatrixXd linpred = ptr->linpred();
+  Eigen::MatrixXd linpred = ptr->matrix.linpred();
   return wrap(linpred);
 }
 
 // [[Rcpp::export]]
 SEXP Model__log_likelihood(SEXP xp){
   XPtr<glmmr::Model> ptr(xp);
-  double logl = ptr->log_likelihood();
+  double logl = ptr->optim.log_likelihood();
   return wrap(logl);
 }
 
 // [[Rcpp::export]]
 void Model__ml_theta(SEXP xp){
   XPtr<glmmr::Model> ptr(xp);
-  ptr->ml_theta();
+  ptr->optim.ml_theta();
 }
 
 // [[Rcpp::export]]
 void Model__ml_beta(SEXP xp){
   XPtr<glmmr::Model> ptr(xp);
-  ptr->ml_beta();
+  ptr->optim.ml_beta();
 }
 
 // [[Rcpp::export]]
 void Model__ml_all(SEXP xp){
   XPtr<glmmr::Model> ptr(xp);
-  ptr->ml_all();
+  ptr->optim.ml_all();
 }
 
 // [[Rcpp::export]]
 void Model__laplace_ml_beta_u(SEXP xp){
   XPtr<glmmr::Model> ptr(xp);
-  ptr->laplace_ml_beta_u();
+  ptr->optim.laplace_ml_beta_u();
 }
 
 // [[Rcpp::export]]
 void Model__laplace_ml_theta(SEXP xp){
   XPtr<glmmr::Model> ptr(xp);
-  ptr->laplace_ml_theta();
+  ptr->optim.laplace_ml_theta();
 }
 
 // [[Rcpp::export]]
 void Model__laplace_ml_beta_theta(SEXP xp){
   XPtr<glmmr::Model> ptr(xp);
-  ptr->laplace_ml_beta_theta();
+  ptr->optim.laplace_ml_beta_theta();
 }
 
 // [[Rcpp::export]]
 void Model__nr_beta(SEXP xp){
   XPtr<glmmr::Model> ptr(xp);
-  ptr->nr_beta();
+  ptr->optim.nr_beta();
 }
 
 // [[Rcpp::export]]
 void Model__laplace_nr_beta_u(SEXP xp){
   XPtr<glmmr::Model> ptr(xp);
-  ptr->laplace_nr_beta_u();
+  ptr->optim.laplace_nr_beta_u();
 }
 
 // [[Rcpp::export]]
 SEXP Model__Sigma(SEXP xp, bool inverse){
   XPtr<glmmr::Model> ptr(xp);
-  Eigen::MatrixXd S = ptr->Sigma(inverse);
+  Eigen::MatrixXd S = ptr->matrix.Sigma(inverse);
   return wrap(S);
 }
 
 // [[Rcpp::export]]
 SEXP Model__information_matrix(SEXP xp){
   XPtr<glmmr::Model> ptr(xp);
-  Eigen::MatrixXd M = ptr->information_matrix();
+  Eigen::MatrixXd M = ptr->matrix.information_matrix();
   return wrap(M);
 }
 
 // [[Rcpp::export]]
 SEXP Model__obs_information_matrix(SEXP xp){
   XPtr<glmmr::Model> ptr(xp);
-  MatrixXd infomat = ptr->observed_information_matrix();
+  MatrixXd infomat = ptr->matrix.observed_information_matrix();
   return wrap(infomat);
 }
 
 // [[Rcpp::export]]
 SEXP Model__u(SEXP xp, bool scaled_){
   XPtr<glmmr::Model> ptr(xp);
-  Eigen::MatrixXd u = ptr->u(scaled_);
+  Eigen::MatrixXd u = ptr->re.u(scaled_);
   return wrap(u);
 }
 
 // [[Rcpp::export]]
 SEXP Model__Zu(SEXP xp){
   XPtr<glmmr::Model> ptr(xp);
-  Eigen::MatrixXd Zu = ptr->Zu();
+  Eigen::MatrixXd Zu = ptr->re.Zu();
   return wrap(Zu);
 }
 
 // [[Rcpp::export]]
 SEXP Model__P(SEXP xp){
   XPtr<glmmr::Model> ptr(xp);
-  int u = ptr->P_;
+  int u = ptr->model.linear_predictor.P();
   return wrap(u);
 }
 
 // [[Rcpp::export]]
 SEXP Model__Q(SEXP xp){
   XPtr<glmmr::Model> ptr(xp);
-  int Q = ptr->Q_;
+  int Q = ptr->model.covariance.Q();
   return wrap(Q);
 }
 
 // [[Rcpp::export]]
 SEXP Model__X(SEXP xp){
   XPtr<glmmr::Model> ptr(xp);
-  Eigen::MatrixXd X = ptr->linpred_.X();
+  Eigen::MatrixXd X = ptr->model.linear_predictor.X();
   return wrap(X);
 }
 
@@ -211,7 +243,7 @@ void Model__mcmc_sample(SEXP xp, SEXP warmup_, SEXP samples_, SEXP adapt_){
   int samples = as<int>(samples_);
   int adapt = as<int>(adapt_);
   XPtr<glmmr::Model> ptr(xp);
-  ptr->mcmc_sample(warmup,samples,adapt);
+  ptr->mcmc.mcmc_sample(warmup,samples,adapt);
 }
 
 // [[Rcpp::export]]
@@ -224,35 +256,35 @@ void Model__set_trace(SEXP xp, SEXP trace_){
 // [[Rcpp::export]]
 SEXP Model__get_beta(SEXP xp){
   XPtr<glmmr::Model> ptr(xp);
-  Eigen::VectorXd beta = ptr->linpred_.parameter_vector();
+  Eigen::VectorXd beta = ptr->model.linear_predictor.parameter_vector();
   return wrap(beta);
 }
 
 // [[Rcpp::export]]
 SEXP Model__y(SEXP xp){
   XPtr<glmmr::Model> ptr(xp);
-  Eigen::VectorXd y = ptr->y_;
+  Eigen::VectorXd y = ptr->model.data.y;
   return wrap(y);
 }
 
 // [[Rcpp::export]]
 SEXP Model__get_theta(SEXP xp){
   XPtr<glmmr::Model> ptr(xp);
-  std::vector<double> theta = ptr->covariance_.parameters_;
+  std::vector<double> theta = ptr->model.covariance.parameters_;
   return wrap(theta);
 }
 
 // [[Rcpp::export]]
 SEXP Model__get_var_par(SEXP xp){
   XPtr<glmmr::Model> ptr(xp);
-  double theta = ptr->var_par_;
+  double theta = ptr->model.data.var_par;
   return wrap(theta);
 }
 
 // [[Rcpp::export]]
 SEXP Model__get_variance(SEXP xp){
   XPtr<glmmr::Model> ptr(xp);
-  Eigen::ArrayXd theta = ptr->variance_;
+  Eigen::ArrayXd theta = ptr->model.data.variance;
   return wrap(theta);
 }
 
@@ -260,36 +292,36 @@ SEXP Model__get_variance(SEXP xp){
 void Model__set_var_par(SEXP xp, SEXP var_par_){
   double var_par = as<double>(var_par_);
   XPtr<glmmr::Model> ptr(xp);
-  ptr->update_var_par(var_par);
+  ptr->model.data.set_var_par(var_par);
 }
 
 // [[Rcpp::export]]
 void Model__set_trials(SEXP xp, SEXP trials){
   Eigen::ArrayXd var_par = as<Eigen::ArrayXd>(trials);
   XPtr<glmmr::Model> ptr(xp);
-  if(ptr->family_ != "binomial")Rcpp::stop("trials can only be set for binomial family.");
-  if(var_par.size()!=ptr->n_)Rcpp::stop("trials wrong length");
-  ptr->update_var_par(var_par);
+  if(ptr->model.family.family != "binomial")Rcpp::stop("trials can only be set for binomial family.");
+  if(var_par.size()!=ptr->model.n())Rcpp::stop("trials wrong length");
+  ptr->model.data.set_variance(var_par);
 }
 
 // [[Rcpp::export]]
 SEXP Model__L(SEXP xp){
   XPtr<glmmr::Model> ptr(xp);
-  Eigen::MatrixXd L = ptr->covariance_.D(true,false);
+  Eigen::MatrixXd L = ptr->model.covariance.D(true,false);
   return wrap(L);
 }
 
 // [[Rcpp::export]]
 SEXP Model__ZL(SEXP xp){
   XPtr<glmmr::Model> ptr(xp);
-  Eigen::MatrixXd ZL = ptr->covariance_.ZL();
+  Eigen::MatrixXd ZL = ptr->model.covariance.ZL();
   return wrap(ZL);
 }
 
 // [[Rcpp::export]]
 SEXP Model__xb(SEXP xp){
   XPtr<glmmr::Model> ptr(xp);
-  Eigen::VectorXd xb = ptr->xb();
+  Eigen::VectorXd xb = ptr->model.xb();
   return wrap(xb);
 }
 
